@@ -3,7 +3,7 @@ package Net::LibNFS;
 use strict;
 use warnings;
 
-our $VERSION = '0.01_01';
+our $VERSION = '0.04_90';
 
 =encoding utf-8
 
@@ -36,7 +36,7 @@ Blocking I/O:
         print "Name: ", $dir_obj->name(), $/;
     }
 
-    $nfs->unmount();
+    $nfs->umount();
 
 Non-blocking I/O, using L<IO::Async>:
 
@@ -51,7 +51,7 @@ Non-blocking I/O, using L<IO::Async>:
             print "Name: ", $dir_obj->name(), $/;
         }
     } )->then( sub {
-        $nfsa->unmount();
+        $nfsa->umount();
     } )->finally( sub { $loop->stop() } );
 
     $loop->run();
@@ -67,8 +67,17 @@ can’t (or just would rather not) L<mount(8)> them locally.
 =head1 LINKING
 
 If a shared libnfs is available and is version 5.0.0 or later we’ll
-compile against that.
-Otherwise we try to compile our own libnfs and link to it statically.
+link (dynamically) against that.
+Otherwise we try to compile our own libnfs and bundle it (statically).
+
+A shared libnfs is usually preferable because it can receive updates on its
+own; a static libnfs locks you into that version of the library until you
+rebuild Net::LibNFS.
+
+If you have a usable shared libnfs but for some reason still want
+to bundle a custom-built static one, set C<NET_LIBNFS_LINK_STATIC> to a
+truthy value in the environment as you run this distribution’s
+F<Makefile.PL>.
 
 =head1 CHARACTER ENCODING
 
@@ -126,7 +135,8 @@ A setter for multiple settings; e.g., where libnfs exposes
 C<nfs_set_version()>, here you pass C<version> with the appropriate
 value.
 
-Recognized options are:
+Recognized options are as follows. (Some may be unavailable if you
+use an older, shared libnfs.)
 
 =over
 
@@ -134,13 +144,21 @@ Recognized options are:
 
 =item * C<client_name> and C<verifier> (NFSv4 only)
 
-=item * C<tcp_syncnt>, C<uid>, C<gid>, C<debug>, C<dircache>,
-C<autoreconnect>, C<timeout>
+=item * C<tcp_syncnt>, C<uid>, C<gid>, C<debug>, C<auto_traverse_mounts>,
+C<dircache>, C<autoreconnect>, C<timeout>, C<nfsport>, C<mountport>
+
+=item * C<unix_authn> (arrayref) - Sets UID, GID, and auxiliary GIDs
+at once. Clobbers (and is clobbered by) C<uid> and C<gid>.
 
 =item * C<pagecache>, C<pagecache_ttl>, C<readahead>
 
 =item * C<readmax>, C<writemax>
 
+=item * C<readdir_buffer> - Sets the maximum buffer size for C<READDIRPLUS>
+(which is used by I<OBJ>->opendir). Can be a two-element arrayref to set
+C<dircount> and C<maxcount> independently or an unsigned integer to set both
+to the same value.
+
 =back
 
 =head2 $old_umask = I<OBJ>->umask( $NEW_UMASK )
